%
% Script for the Monte-Carlo integration of the parameter volume.
%
% Model should be renamed as 'oModel'. Starting sampled parameters should be 
% rename as 'goodks'. PromptMode=0 for no interface.
% Parameters for simulations are (used as defaults if already exist):
%   - 'simulationnumber' , number for autobackup
%   - 'kcheckfunction' , handle to function(oModel, vk [, mainX0])
%           returning a logical value. Name should be kcheck_modelname*.m
%           where modelname is the model used and * can be anything.
%   - 'mainX0' , initial conditions for kcheckfunction
%   - 'nbsets' , # of parameters tested per step
%   - 'nbhits' , # of maximal viable parameter per step
%
% This function does not store the rejected parameters --> allows to test
% more parameters and faster --> set nbsets rather high such that nbhits is
% reached.
%
% See also MULTIMC_ALONG_PCA, VOLUMEMC_ALONG_PCA.
%
% Version 1, october 2008
% Marc Hafner, EPFL-IC-LANOS, Switzerland
% help and comments to marc.hafner@epfl.ch
%

%% needed variables

functionname = 'volumeMC_along_PCA2';
eval(sprintf('i0name = ''%si0'';',functionname));

if exist('oModel','var')==0
    disp('Rename the model to be used as ''oModel'' ')
    return;
else
    disp(['Model used: ' oModel.tModelName])
end

if exist('goodks','var')==0
    disp('Rename the initial sets to be used as ''goodks'' ')
    return;
end

global PromptMode
if isempty(PromptMode)==1
    displayPromptModes();
    PromptMode = input('Please set PromptMode, 0-1: ');
end

global DisplayMode
if isempty(DisplayMode)==1
    displayDisplayModes();
    DisplayMode = input('Please set DisplayMode, 0-5: ');
end

SelectWorkingPath

%% checking for old simulations


skip=0;
if exist(i0name,'var')
    if PromptMode; reply = input('On going simulation, do you want to continue it y/n [n]? ', 's');
    else reply='n';end;
    if isempty(reply)==1 || reply=='N' || reply=='n'
        clear(i0name);
    else
        skip=1;
    end
end

if ~exist([bioSimworkingpath filesep 'backup' filesep],'dir')
    mkdir([bioSimworkingpath filesep 'backup' filesep])
end

if skip==0

    if ~exist('simulationnumber','var') || floor(simulationnumber)~=simulationnumber;simulationnumber = 1;end;      % simulation number
    if PromptMode; reply = input(['Simulation number ' num2str(simulationnumber) ', do you agree y/1-.../n [y]? '], 's');
    else reply='y';end;

    if ~isempty(reply) && ~isempty(str2double(reply))
        simulationnumber = floor(str2double(reply));
    elseif isempty(reply)==0 && reply(1)~='Y' && reply(1)~='y'
        simulationnumber = floor(input('Choose simulation number: '));
    end

    tempname = sprintf('%s%sbackup%s%s_data_%s_temp%i.mat',bioSimworkingpath,filesep,filesep,functionname,oModel.tModelName,simulationnumber);
    
    if exist(tempname,'file')
        disp(['found old simulation: ' tempname])
        if PromptMode; reply = input('Do you want to load these data y/n [y]? ', 's');
        else reply='y';end;
        if isempty(reply)==1 || reply=='Y' || reply=='y'
            load(tempname)
            disp([tempname ' loaded'])
        end
    end
end


%% a priori parameter conditions
if skip==0 || exist('kcheckfunction','var')==0

    ckfunctions = dir(['kcheck_' oModel.tModelName '*.m']);

    if ~isempty(ckfunctions)
        if PromptMode;
            disp('Aviable function for conditions on parameter:');
            for i=1:length(ckfunctions)
                disp(['    ' num2str(i) '    ' ckfunctions(i).name]);
            end
            reply = input(['Choose which function do you want to use 1-' num2str(length(ckfunctions)) ' [1]? '], 's');
        else reply='1';
        end

        if isempty(reply), reply='1'; end;
        ckfunctions = ckfunctions(str2num(reply)).name;
        pointposition = max( (ckfunctions=='.') .* [1:length(ckfunctions)] );
        eval(sprintf('kcheckfunction = @%s;', ckfunctions(1:pointposition-1) ));

        if nargin(kcheckfunction)~=2 && nargin(kcheckfunction)~=3
            error(['Inputs for function with conditions on parameter found should be (oModel, vk [, vX0])'])
        end
        if nargout(kcheckfunction)~=3
            error(['Inputs for function with conditions on parameter found should be (oModel, vk [, vX0])' ...
                '; outputs should be [test, X0, conditions]'])
        end
        
    else
        file = fopen(['kcheck_' oModel.tModelName '0.m'],'w');
        fprintf(file, ['function [test, X0, conditions] = kcheck_%s0 (oModel, vk, vX0)\n\n' ...
            '%% Enter the condition for vk (test = true or false)\n' ...
            '%% Enter an output ( X0 = state vector)\n' ...
            '%% Enter a condition vector (conditions = display output)\n\n' ], oModel.tModelName);
        fclose(file);
        open(['kcheck_' oModel.tModelName '0.m']);
        error(['No function with conditions on parameter found. Create a m file kcheck_' ...
            oModel.tModelName '*.m in the current folder. Inputs should be (oModel, vk [, vX0])' ...
            '; outputs should be [test, X0, conditions]'])
    end
end


%% initial conditions
if skip==0 && ~exist('mainX0','var') && nargin(kcheckfunction)>2
    disp([func2str(kcheckfunction) ' required initial conditions'])
    if PromptMode
        disp('No initial conditions specified')
        disp('Model default initial conditions are:')
        disp(oModel.vX0)
        reply = input('Do you want to use the model default initial conditions [y]/n?','s');
    else reply='y';
    end
    
    if ~isempty(reply) && (reply=='N' || reply=='n')
        error('Specify the initial conditions in vector ''mainX0''');
    else
        mainX0 = oModel.vX0;
    end
end



%% define the parameters of the simulation

disp('Used settings: ')

if ~exist('nbsets','var');nbsets = 1e8;end;       % nbre of tested set per step
disp(['   nbre of tested sets: ' num2str(nbsets)])

if ~exist('nbhits','var');nbhits = 500;end;      % nbre of max goodks per step
disp(['   max nbre of good steps: ' num2str(nbhits)])


disp(['Using a priori conditions of function: ' func2str(kcheckfunction)])

if nargin(kcheckfunction)>2
    disp('Using initial conditions')
    disp(mainX0)
end

disp(['Simulation number : ' num2str(simulationnumber)])

if PromptMode; reply = input('Do you want to perform the MC integration with these settings y/n [y]? ', 's');
else reply='y';end;
if isempty(reply)==0 && (reply=='N' || reply=='n')
    return
end
disp([' '])


%% initialization part
time=clock;
rand('twister',1e7*simulationnumber+time(6)+100*time(5)+1e4*time(4)+1e6*time(3));
randn('seed',1e7*simulationnumber+time(6)+100*time(5)+1e4*time(4)+1e6*time(3));

eval(sprintf('logbook = fopen(''logbook_%s_%s_%i.txt'',''a'');',functionname,oModel.tModelName,simulationnumber));

fprintf(logbook,'\n ---- Day (MMDD) : %i/%i Time : %i:%i ----\n\n',time(2),time(3), time(4), time(5));

fprintf(logbook,'%s\n',['   nbre of tested sets: ' num2str(nbsets)]);
fprintf(logbook,'%s\n',['   max nbre of good steps per step: ' num2str(nbhits)]);
fprintf(logbook,'using conditions function on the parameters: %s\n',func2str(kcheckfunction));
if nargin(kcheckfunction)>2
    fprintf(logbook,'Initial conditions: ');
    for i=1:length(mainX0)
        fprintf(logbook,'%.3e ',mainX0(i));
    end
    fprintf(logbook,'\n');
end


lk = oModel.slk - oModel.sconstrained;
lX = oModel.slX;
k0 = oModel.vk;
cc = k0(end-oModel.sconstrained+1:end);
k0 = k0(1:end-oModel.sconstrained);
ksign=sign(k0);


%% major loop

time=clock;
fprintf(logbook,'\n ---- Day (MMDD) : %i/%i Time : %i:%i ----\n\n',time(2),time(3), time(4), time(5));

if exist(i0name,'var')==0
    vark = var(goodks);
    vark = sort((vark~=0).*[1:length(vark)],'descend');
    vark = sort(vark(1:max((vark~=0).*[1:length(vark)])));

    logks = log10ks(goodks(:,vark));
    
    [axis2, coord, latent] = princomp(logks);
    axis = zeros(lk,lk);
    axis(vark,vark) = axis2;
    variance = zeros(lk,1);
    variance(vark) = sqrt(latent);
    lastlogmean = -Inf*ones(1,lk);
    lastlogmean(vark) = mean(logks);

    maxcoord = zeros(1,lk);
    maxcoord(vark) = max(coord);
    mincoord = zeros(1,lk);
    mincoord(vark) = min(coord);

    V0 = prod(maxcoord(vark)-mincoord(vark));
    reallk = length(vark);

    
    % test the new sets for viable results

    data = simulation_deterministic(oModel,[0 500], [(10.^lastlogmean) cc], oModel.vX0);
    X0=data.X;
    X0=X0(end,:);

    clear newgoods
    eval(sprintf('%s=1;',i0name));
    interni0=1;
    newgoods = zeros(0,lk);
else
    eval(sprintf('interni0 = %s;',i0name));
end

wbar = waitbar(0,['Simulation: ' functionname ' ' oModel.tModelName ' sim n' num2str(simulationnumber)]);

for i=interni0:nbsets
    if mod(i,100)==0;
        fprintf('set %i (%i good ones)\n ',i,size(newgoods,1)); 
        waitbar(i/nbsets);
    end
    
    % generate the parameters sets

    % sets are equvalently distribute along the major axis in the min/max
    % interval
    positions = (rand(1,lk)) .* (maxcoord-mincoord) + mincoord;

    sets = (axis*positions')';
    sets = sets + lastlogmean(1:lk);
    sets = (10.^sets).*ksign(1:lk);

    if nargin(kcheckfunction)>2
        eval(sprintf('test = kcheckfunction(oModel, [sets cc], mainX0);'));
    else
        eval(sprintf('test = kcheckfunction(oModel, [sets cc]);'));
    end

    if test

        if exist('newgoods','var')==0
            newgoods = [sets cc];
        else
            newgoods(end+1,:) = [sets cc];
        end

        if length(newgoods)>=nbhits
            break
        end

    else
        if DisplayMode>4; fprintf('set %i refused due to kcheckfunction\n ',i); end
    end

    if mod(i,1e4)==900
        eval(sprintf('%s = i+1;',i0name));
        fprintf('Saving at set %i (%i good ones)',i,size(newgoods,1));
        fprintf(logbook,'Saving at set: %i;\n', i);
        if exist('newgoods')
            save(tempname, 'newgoods', i0name, 'mainX0', 'lastlogmean', 'kcheckfunction', 'mincoord', 'maxcoord', 'axis', 'vark', 'V0')
        else
            save(tempname, i0name, 'mainX0', 'kcheckfunction', 'mincoord', 'maxcoord', 'axis', 'vark', 'V0', 'lastlogmean')
        end

    end

end
close(wbar)
%% results
eval(sprintf('nsetsmax = %s',i0name));

Volume = V0*size(newgoods,1)/nsetsmax
NormilzedVolume =  Volume^(1/length(vark))

eval(sprintf('save ./%s_results_%s_%i.mat nsetsmax newgoods kcheckfunction mincoord maxcoord vark axis NormilzedVolume',functionname,oModel.tModelName,simulationnumber));
delete(tempname)
clear volMCi0

time=clock;
fprintf(' ; loop over, data saved \n');
fprintf(logbook,'\nloop over, data saved \n');

fprintf(logbook,'\n ---- Day (MMDD) : %i/%i Time : %i:%i ----\n\n',time(2),time(3), time(4), time(5));
fclose(logbook)


%% plotting both distributions

plotks(sets,'.k')
hold on
plotks(newgoods(:,1:(end-length(cc))),'.b')
hold off


